from .base import BaseAdapter
from .alice import AliceAdapter
from .facebook import FacebookAdapter
from .text import TextAdapter
from .tg import TelegramAdapter
from .vk import VkAdapter
from .salut import SalutAdapter
